@@ -55,17 +55,18 @@ static const int NCPUS_START = sizeof(unsigned long) * CHAR_BIT;
 #endif
 
 
-#if PSUTIL_HAVE_IOPRIO
-enum {
-    IOPRIO_WHO_PROCESS = 1,
-};
-
 // May happen on old RedHat versions, see:
 // https://github.com/giampaolo/psutil/issues/607
 #ifndef DUPLEX_UNKNOWN
     #define DUPLEX_UNKNOWN 0xff
 #endif
 
+
+#if PSUTIL_HAVE_IOPRIO
+enum {
+    IOPRIO_WHO_PROCESS = 1,
+};
+
 static inline int
 ioprio_get(int which, int who) {
     return syscall(__NR_ioprio_get, which, who);
@@ -246,13 +247,15 @@ psutil_linux_sysinfo(PyObject *self, PyObject *args) {
         return PyErr_SetFromErrno(PyExc_OSError);
     // note: boot time might also be determined from here
     return Py_BuildValue(
-        "(KKKKKK)",
-        (unsigned long long)info.totalram  * info.mem_unit,   // total
-        (unsigned long long)info.freeram   * info.mem_unit,   // free
-        (unsigned long long)info.bufferram * info.mem_unit,   // buffer
-        (unsigned long long)info.sharedram * info.mem_unit,   // shared
-        (unsigned long long)info.totalswap * info.mem_unit,   // swap tot
-        (unsigned long long)info.freeswap  * info.mem_unit);  // swap free
+        "(kkkkkkI)",
+        info.totalram,  // total
+        info.freeram,  // free
+        info.bufferram, // buffer
+        info.sharedram, // shared
+        info.totalswap, // swap tot
+        info.freeswap,  // swap free
+        info.mem_unit  // multiplier
+    );
 }
 
 
@@ -363,7 +366,7 @@ psutil_proc_cpu_affinity_get(PyObject *self, PyObject *args) {
 
 error:
     Py_XDECREF(py_cpu_num);
-    Py_DECREF(py_retlist);
+    Py_XDECREF(py_retlist);
     return NULL;
 }
 #endif
