@@ -88,12 +88,14 @@ psutil_file_to_struct(char *path, void *fstruct, size_t size) {
 static PyObject *
 psutil_proc_basic_info(PyObject *self, PyObject *args) {
     int pid;
-    char path[100];
+    char path[1000];
     psinfo_t info;
+    const char *procfs_path;
 
-    if (! PyArg_ParseTuple(args, "i", &pid))
+    if (! PyArg_ParseTuple(args, "is", &pid, &procfs_path))
         return NULL;
-    sprintf(path, "/proc/%i/psinfo", pid);
+
+    sprintf(path, "%s/%i/psinfo", procfs_path, pid);
     if (! psutil_file_to_struct(path, (void *)&info, sizeof(info)))
         return NULL;
     return Py_BuildValue("ikkdiiik",
@@ -115,15 +117,29 @@ psutil_proc_basic_info(PyObject *self, PyObject *args) {
 static PyObject *
 psutil_proc_name_and_args(PyObject *self, PyObject *args) {
     int pid;
-    char path[100];
+    char path[1000];
     psinfo_t info;
+    const char *procfs_path;
+    PyObject *py_name;
+    PyObject *py_args;
 
-    if (! PyArg_ParseTuple(args, "i", &pid))
+    if (! PyArg_ParseTuple(args, "is", &pid, &procfs_path))
         return NULL;
-    sprintf(path, "/proc/%i/psinfo", pid);
+    sprintf(path, "%s/%i/psinfo", procfs_path, pid);
     if (! psutil_file_to_struct(path, (void *)&info, sizeof(info)))
         return NULL;
+
+#if PY_MAJOR_VERSION >= 3
+    py_name = PyUnicode_DecodeFSDefault(info.pr_fname);
+    if (!py_name)
+        return NULL;
+    py_args = PyUnicode_DecodeFSDefault(info.pr_psargs);
+    if (!py_args)
+        return NULL;
+    return Py_BuildValue("OO", py_name, py_args);
+#else
     return Py_BuildValue("ss", info.pr_fname, info.pr_psargs);
+#endif
 }
 
 
@@ -133,12 +149,13 @@ psutil_proc_name_and_args(PyObject *self, PyObject *args) {
 static PyObject *
 psutil_proc_cpu_times(PyObject *self, PyObject *args) {
     int pid;
-    char path[100];
+    char path[1000];
     pstatus_t info;
+    const char *procfs_path;
 
-    if (! PyArg_ParseTuple(args, "i", &pid))
+    if (! PyArg_ParseTuple(args, "is", &pid, &procfs_path))
         return NULL;
-    sprintf(path, "/proc/%i/status", pid);
+    sprintf(path, "%s/%i/status", procfs_path, pid);
     if (! psutil_file_to_struct(path, (void *)&info, sizeof(info)))
         return NULL;
     // results are more precise than os.times()
@@ -154,12 +171,13 @@ psutil_proc_cpu_times(PyObject *self, PyObject *args) {
 static PyObject *
 psutil_proc_cred(PyObject *self, PyObject *args) {
     int pid;
-    char path[100];
+    char path[1000];
     prcred_t info;
+    const char *procfs_path;
 
-    if (! PyArg_ParseTuple(args, "i", &pid))
+    if (! PyArg_ParseTuple(args, "is", &pid, &procfs_path))
         return NULL;
-    sprintf(path, "/proc/%i/cred", pid);
+    sprintf(path, "%s/%i/cred", procfs_path, pid);
     if (! psutil_file_to_struct(path, (void *)&info, sizeof(info)))
         return NULL;
     return Py_BuildValue("iiiiii",
@@ -174,12 +192,13 @@ psutil_proc_cred(PyObject *self, PyObject *args) {
 static PyObject *
 psutil_proc_num_ctx_switches(PyObject *self, PyObject *args) {
     int pid;
-    char path[100];
+    char path[1000];
     prusage_t info;
+    const char *procfs_path;
 
-    if (! PyArg_ParseTuple(args, "i", &pid))
+    if (! PyArg_ParseTuple(args, "is", &pid, &procfs_path))
         return NULL;
-    sprintf(path, "/proc/%i/usage", pid);
+    sprintf(path, "%s/%i/usage", procfs_path, pid);
     if (! psutil_file_to_struct(path, (void *)&info, sizeof(info)))
         return NULL;
     return Py_BuildValue("kk", info.pr_vctx, info.pr_ictx);
@@ -200,12 +219,13 @@ psutil_proc_num_ctx_switches(PyObject *self, PyObject *args) {
 static PyObject*
 proc_io_counters(PyObject* self, PyObject* args) {
     int pid;
-    char path[100];
+    char path[1000];
     prusage_t info;
+    const char *procfs_path;
 
-    if (! PyArg_ParseTuple(args, "i", &pid))
+    if (! PyArg_ParseTuple(args, "is", &pid, &procfs_path))
         return NULL;
-    sprintf(path, "/proc/%i/usage", pid);
+    sprintf(path, "%s/%i/usage", procfs_path, pid);
     if (! psutil_file_to_struct(path, (void *)&info, sizeof(info)))
         return NULL;
 
@@ -229,12 +249,13 @@ proc_io_counters(PyObject* self, PyObject* args) {
 static PyObject *
 psutil_proc_query_thread(PyObject *self, PyObject *args) {
     int pid, tid;
-    char path[100];
+    char path[1000];
     lwpstatus_t info;
+    const char *procfs_path;
 
-    if (! PyArg_ParseTuple(args, "ii", &pid, &tid))
+    if (! PyArg_ParseTuple(args, "iis", &pid, &tid, &procfs_path))
         return NULL;
-    sprintf(path, "/proc/%i/lwp/%i/lwpstatus", pid, tid);
+    sprintf(path, "%s/%i/lwp/%i/lwpstatus", procfs_path, pid, tid);
     if (! psutil_file_to_struct(path, (void *)&info, sizeof(info)))
         return NULL;
     return Py_BuildValue("dd",
@@ -545,7 +566,7 @@ static PyObject *
 psutil_proc_memory_maps(PyObject *self, PyObject *args) {
     int pid;
     int fd = -1;
-    char path[100];
+    char path[1000];
     char perms[10];
     char *name;
     struct stat st;
@@ -557,20 +578,21 @@ psutil_proc_memory_maps(PyObject *self, PyObject *args) {
     int nmap;
     uintptr_t pr_addr_sz;
     uintptr_t stk_base_sz, brk_base_sz;
+    const char *procfs_path;
 
     PyObject *py_tuple = NULL;
     PyObject *py_retlist = PyList_New(0);
 
     if (py_retlist == NULL)
         return NULL;
-    if (! PyArg_ParseTuple(args, "i", &pid))
+    if (! PyArg_ParseTuple(args, "is", &pid, &procfs_path))
         goto error;
 
-    sprintf(path, "/proc/%i/status", pid);
+    sprintf(path, "%s/%i/status", procfs_path, pid);
     if (! psutil_file_to_struct(path, (void *)&status, sizeof(status)))
         goto error;
 
-    sprintf(path, "/proc/%i/xmap", pid);
+    sprintf(path, "%s/%i/xmap", procfs_path, pid);
     if (stat(path, &st) == -1) {
         PyErr_SetFromErrno(PyExc_OSError);
         goto error;
