@@ -19,7 +19,9 @@
 #include <sys/sysctl.h>
 #include <sys/user.h>
 #include <sys/proc.h>
+#include <sys/mount.h>  // for VFS_*
 #include <sys/swap.h>  // for swap_mem
+#include <sys/vmmeter.h>  // for vmtotal struct
 #include <signal.h>
 #include <kvm.h>
 // connection stuff
@@ -38,7 +40,8 @@
 #include "../../_psutil_common.h"
 
 #define PSUTIL_KPT2DOUBLE(t) (t ## _sec + t ## _usec / 1000000.0)
-#define PSUTIL_TV2DOUBLE(t) ((t).tv_sec + (t).tv_usec / 1000000.0)
+// #define PSUTIL_TV2DOUBLE(t) ((t).tv_sec + (t).tv_usec / 1000000.0)
+
 // a signaler for connections without an actual status
 int PSUTIL_CONN_NONE = 128;
 
@@ -95,7 +98,7 @@ kinfo_getfile(long pid, int* cnt) {
         return NULL;
     }
     if ((kf = malloc(len)) == NULL) {
-        PyErr_SetFromErrno(PyExc_OSError);
+        PyErr_NoMemory();
         return NULL;
     }
     mib[5] = (int)(len / sizeof(struct kinfo_file));
@@ -176,6 +179,7 @@ psutil_get_proc_list(struct kinfo_proc **procList, size_t *procCount) {
 
     result = kvm_getprocs(kd, KERN_PROC_ALL, 0, sizeof(struct kinfo_proc), &cnt);
     if (result == NULL) {
+        kvm_close(kd);
         err(1, NULL);
         return errno;
     }
@@ -185,6 +189,7 @@ psutil_get_proc_list(struct kinfo_proc **procList, size_t *procCount) {
     size_t mlen = cnt * sizeof(struct kinfo_proc);
 
     if ((*procList = malloc(mlen)) == NULL) {
+        kvm_close(kd);
         err(1, NULL);
         return errno;
     }
@@ -235,7 +240,11 @@ psutil_get_cmdline(long pid) {
         goto error;
 
     for (p = argv; *p != NULL; p++) {
+#if PY_MAJOR_VERSION >= 3
+        py_arg = PyUnicode_DecodeFSDefault(*p);
+#else
         py_arg = Py_BuildValue("s", *p);
+#endif
         if (!py_arg)
             goto error;
         if (PyList_Append(py_retlist, py_arg))
@@ -321,27 +330,54 @@ error:
 
 PyObject *
 psutil_virtual_mem(PyObject *self, PyObject *args) {
-    unsigned int   total;
-    size_t         size = sizeof(total);
-    struct uvmexp  uvmexp;
-    int            mib[] = {CTL_VM, VM_UVMEXP};
-    long           pagesize = getpagesize();
-    size = sizeof(uvmexp);
+    int64_t total_physmem;
+    int uvmexp_mib[] = {CTL_VM, VM_UVMEXP};
+    int bcstats_mib[] = {CTL_VFS, VFS_GENERIC, VFS_BCACHESTAT};
+    int physmem_mib[] = {CTL_HW, HW_PHYSMEM64};
+    int vmmeter_mib[] = {CTL_VM, VM_METER};
+    size_t size;
+    struct uvmexp uvmexp;
+    struct bcachestats bcstats;
+    struct vmtotal vmdata;
+    long pagesize = getpagesize();
 
-    if (sysctl(mib, 2, &uvmexp, &size, NULL, 0) < 0) {
-        warn("failed to get vm.uvmexp");
+    size = sizeof(total_physmem);
+    if (sysctl(physmem_mib, 2, &total_physmem, &size, NULL, 0) < 0) {
         PyErr_SetFromErrno(PyExc_OSError);
         return NULL;
     }
+
+    size = sizeof(uvmexp);
+    if (sysctl(uvmexp_mib, 2, &uvmexp, &size, NULL, 0) < 0) {
+        PyErr_SetFromErrno(PyExc_OSError);
+        return NULL;
+    }
+
+    size = sizeof(bcstats);
+    if (sysctl(bcstats_mib, 3, &bcstats, &size, NULL, 0) < 0) {
+        PyErr_SetFromErrno(PyExc_OSError);
+        return NULL;
+    }
+
+    size = sizeof(vmdata);
+    if (sysctl(vmmeter_mib, 2, &vmdata, &size, NULL, 0) < 0) {
+        PyErr_SetFromErrno(PyExc_OSError);
+        return NULL;
+    }
+
     return Py_BuildValue("KKKKKKKK",
-        (unsigned long long) uvmexp.npages    * pagesize,
-        (unsigned long long) uvmexp.free     * pagesize,
-        (unsigned long long) uvmexp.active   * pagesize,
+        // Note: many programs calculate total memory as
+        // "uvmexp.npages * pagesize" but this is incorrect and does not
+        // match "sysctl | grep hw.physmem".
+        (unsigned long long) total_physmem,
+        (unsigned long long) uvmexp.free * pagesize,
+        (unsigned long long) uvmexp.active * pagesize,
         (unsigned long long) uvmexp.inactive * pagesize,
-        (unsigned long long) uvmexp.wired    * pagesize,
-        (unsigned long long) 0,
-        (unsigned long long) 0,
-        (unsigned long long) 0
+        (unsigned long long) uvmexp.wired * pagesize,
+        // this is how "top" determines it
+        (unsigned long long) bcstats.numbufpages * pagesize,  // cached
+        (unsigned long long) 0,  // buffers
+        (unsigned long long) vmdata.t_vmshr + vmdata.t_rmshr  // shared
     );
 }
 
@@ -358,14 +394,13 @@ psutil_swap_mem(PyObject *self, PyObject *args) {
     }
 
     if ((swdev = calloc(nswap, sizeof(*swdev))) == NULL) {
-        PyErr_SetFromErrno(PyExc_OSError);
+        PyErr_NoMemory();
         return NULL;
     }
 
     if (swapctl(SWAP_STATS, swdev, nswap) == -1) {
-        free(swdev);
         PyErr_SetFromErrno(PyExc_OSError);
-        return NULL;
+        goto error;
     }
 
     // Total things up.
@@ -386,6 +421,10 @@ psutil_swap_mem(PyObject *self, PyObject *args) {
                          // swapent struct does not provide any info
                          // about it.
                          0, 0);
+
+error:
+    free(swdev);
+    return NULL;
 }
 
 
@@ -432,7 +471,11 @@ psutil_proc_cwd(PyObject *self, PyObject *args) {
         PyErr_SetFromErrno(PyExc_OSError);
         return NULL;
     }
+#if PY_MAJOR_VERSION >= 3
+    return PyUnicode_DecodeFSDefault(path);
+#else
     return Py_BuildValue("s", path);
+#endif
 }
 
 
@@ -730,15 +773,12 @@ psutil_disk_io_counters(PyObject *self, PyObject *args) {
 
     for (i = 0; i < dk_ndrive; i++) {
         py_disk_info = Py_BuildValue(
-            "(KKKKLL)",
-            stats[i].ds_rxfer,
-            stats[i].ds_wxfer,
-            stats[i].ds_rbytes,
-            stats[i].ds_wbytes,
-            // assume half read - half writes.
-            // TODO: why?
-            (long long) PSUTIL_TV2DOUBLE(stats[i].ds_time) / 2,
-            (long long) PSUTIL_TV2DOUBLE(stats[i].ds_time) / 2);
+            "(KKKK)",
+            stats[i].ds_rxfer,  // num reads
+            stats[i].ds_wxfer,  // num writes
+            stats[i].ds_rbytes,  // read bytes
+            stats[i].ds_wbytes  // write bytes
+        );
         if (!py_disk_info)
             goto error;
         if (PyDict_SetItemString(py_retdict, stats[i].ds_name, py_disk_info))
